package bot.action;

import bot.Weapon;
import bot.agent.BotContext;
import cz.cuni.amis.pogamut.sposh.context.UT2004Context;
import cz.cuni.amis.pogamut.sposh.executor.ActionResult;
import cz.cuni.amis.pogamut.sposh.executor.Param;
import cz.cuni.amis.pogamut.sposh.executor.ParamsAction;
import cz.cuni.amis.pogamut.sposh.executor.PrimitiveInfo;

/**
 * Action ShootPlayerWithWeapon for Yaposh.
 *
 * @author Jimmy
 * @param <CONTEXT> Context class of the action. It's an shared object used by
 * all primitives. it is used as a shared memory and for interaction with the
 * environment.
 */
@PrimitiveInfo(name = "ShootPlayerWithWeapon", description = "Description of ShootPlayerWithWeapon")
public class ShootPlayerWithWeapon<CONTEXT extends BotContext> extends ParamsAction<CONTEXT> {

    /**
     * Constructor of the action, used during automatic instantiation.
     */
    public ShootPlayerWithWeapon(CONTEXT ctx) {
        super(ctx);
    }

    /**
     * Method responsible for initialization of the action. The method can be
     * passed parameters from the plan. Add all desired plan parameters as
     * method parameters, e.g. <tt>public void init({@literal @}Param("$speed")
     * Integer runningSpeed)</tt>.
     */
    public void init(@Param("$weapon") Weapon weapon) {
        ctx.getWeaponry().changeWeapon(weapon.itemType);
    }

    /**
     * Method called during each tick of the logic the action is supposed to
     * run.
     *
     * The method can be passed parameters from the plan, e.g. <tt>public void run({@literal @}Param("$stuckTime")
     * Double stuckTimeSecs)</tt>.
     */
    public ActionResult run(@Param("$weapon") Weapon weapon) {
        if (ctx.getPlayers().canSeePlayers()) {
            ctx.getShoot().shoot(ctx.getPlayers().getNearestVisiblePlayer());
            ctx.isShooting = true;
        } else {
            ctx.getLog().warning("CANNOT START SHOOTING AS THERE IS NO VISIBLE PLAYER!");
            ctx.getShoot().stopShooting();
            ctx.isShooting = false;
        }
        return ActionResult.FINISHED;   
    }

    /**
     * Method called once engine decides that another action should be run,
     * place you cleanup code here.
     *
     * The method can be passed parameters from the plan, e.g. <tt>public void done({@literal @}Param("$notify")
     * Boolean notifyTeam)</tt>.
     */
    public void done() {
    }
}
