package bot.agent;

import bot.messages.TCMyLocation;
import cz.cuni.amis.pogamut.base.communication.worldview.listener.annotation.EventListener;
import cz.cuni.amis.pogamut.base3d.worldview.object.Rotation;
import cz.cuni.amis.pogamut.base3d.worldview.object.Velocity;
import cz.cuni.amis.pogamut.sposh.context.UT2004Context;
import cz.cuni.amis.pogamut.unreal.communication.messages.UnrealId;
import cz.cuni.amis.pogamut.ut2004.agent.module.utils.TabooSet;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.NavigationState;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.UT2004PathAutoFixer;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.communication.messages.ItemType;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Item;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Player;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.PlayerMessage;
import cz.cuni.amis.utils.flag.FlagListener;

/**
 * This serves as a template bot for creating simple 1v1 DeathMatch bot using YaPOSH.
 * 
 * Many primitives are implemented so it is easy to start design the behvior using graphical
 * YaPOSH editor only.
 * 
 * If you are in NetBeans, check Projects/duelbot project/Other sources/duel-bot.lap plan.
 *
 * @author Jimmy
 */
public class BotContext extends UT2004Context<UT2004Bot> {
    
    /** Current item our bot is currently going for */
    public Item targetItem = null;
    /** Target enemy player we are currently shooting at */
    public Player targetPlayer = null;
    
    /** Used to taboo items we were stuck going for or we have picked up recently */
    public TabooSet<Item> tabooItems;

    /**
     * AutoFixer monitors movement of agent and when it detects faulty
     * connection in the waypoints, it removes them from navigation logic.
     */
    UT2004PathAutoFixer autoFixer;
    
    public String state = "";

    public BotContext(UT2004Bot bot, UnrealId tcServerId) {
        super("DuelBotContext", bot);
        // IMPORTANT: Various modules of context must be initialized.
        initialize();
        
        //getTCClient().setServerId(tcServerId);
        
        // INITIALIZE CUSTOM MODULES
        tabooItems = new TabooSet<Item>(bot);
        
        this.getNavigation().addStrongNavigationListener(new FlagListener<NavigationState>() {
            @Override
            public void flagChanged(NavigationState changedValue) {
                switch (changedValue) {
                    case PATH_COMPUTATION_FAILED:
                    case STUCK:
                        if (targetItem != null)
                            tabooItems.add(targetItem, 30);
                    break;
                    case TARGET_REACHED:
                        if (targetItem != null)
                            tabooItems.add(targetItem, 5);                        
                    break;
                }
            }
        });
        
//        this.getWeaponPrefs()
//                .addGeneralPref(ItemType.LIGHTNING_GUN, true)
//                .addGeneralPref(ItemType.SHOCK_RIFLE, true)
//                .addGeneralPref(ItemType.FLAK_CANNON, true)
//                .addGeneralPref(ItemType.MINIGUN, true)                
//                .addGeneralPref(ItemType.LINK_GUN, true)
//                .addGeneralPref(ItemType.ASSAULT_RIFLE, true); 
        
        
    }

    public boolean isShooting;
    
    public ItemType currentWeapon;
    
    void logicBeforePlan() {
        isShooting = info.isShooting();
        currentWeapon = weaponry.getCurrentWeapon().getType();
    }

    void logicAfterPlan() {      
    	communication();
    }
    
    // =================
    // OUTGOING MESSAGES
    // =================
    
    private void communication() {    	
    	log.info("SENDING MY LOCATION");
    	tcClient.sendToTeamOthers(new TCMyLocation(info.getId(), info.getLocation()));
    }
    
    // =================
    // INCOMING MESSAGES
    // =================
    
    @EventListener(eventClass=TCMyLocation.class)
    public void tcMyLocation(TCMyLocation msg) {
    	Player who = players.getPlayer(msg.getWho());
    	
    	Player update = null;
    	
    	if (who == null) {
    		// NOT SEEN YET
    		update = new PlayerMessage(msg.getWho(), "", "N/A", false, "", false, new Rotation(0,0,0), msg.getLocation(), new Velocity(0,0,0), info.getTeam(), "", false, 0, "", "", "", "", "");
    	} else {
    		// ALREADY SEEN
    		if (who.isVisible()) return; 
    		update = new PlayerMessage(who.getId(), who.getJmx(), who.getName(), who.isSpectator(), who.getAction(), who.isVisible(), who.getRotation(), msg.getLocation(), who.getVelocity(), who.getTeam(), who.getWeapon(), who.isCrouched(), who.getFiring(), who.getEmotLeft(), who.getEmotCenter(), who.getEmotRight(), who.getBubble(), who.getAnim());
    	}
    	
    	getWorldView().notifyImmediately(update);
    	
    	log.info("UPDATED: " + msg.getWho().getStringId() + " -> " + msg.getLocation());
    }
}
